package go_swagger

import (
	"os"
	"path/filepath"
	"strings"

	"github.com/go-swagger/go-swagger/cmd/swagger/commands"
	"github.com/go-swagger/go-swagger/cmd/swagger/commands/generate"
	"github.com/jessevdk/go-flags"
	"github.com/juju/errors"

	"git.dev.m-and-m.ovh/mderasse/gouick/helpers"

	log "github.com/sirupsen/logrus"
)

// executeGoSwagger will merge api yaml files and launch go-swagger generator.
func (a APIType) executeGoSwagger(path string) error {
	log.Debugf("Starting %s - %s", a.GetName(), helpers.GetCurrentFuncName())

	// making sure that we are in the project directory
	if err := os.Chdir(path); err != nil {
		log.Error("Fail to move to the path")
		return errors.Trace(err)
	}

	// list files in api/*
	log.Debug("Merging api files")

	subApiFiles, err := filepath.Glob(filepath.Join(path, "api", "*"))
	if err != nil {
		log.Error("Fail to list api files in api directory")
		return errors.Trace(err)
	}

	// merge api files.
	collisions, err := a.goSwaggerMixin(path, "api.yaml", subApiFiles)
	if err != nil {
		log.Error("Fail to merge api files")
		return errors.Trace(err)
	}

	if len(collisions) > 0 {
		return errors.NotValidf("The following collisions has been detected: %s", strings.Join(collisions, "\n"))
	}

	log.Debug("Api files successfully merged")

	// cleanup some directory to avoid issues.
	for _, directory := range []string{"models", "restapi/operations"} {
		err := helpers.RemoveDirectoryContent(filepath.Join(path, directory))
		if err != nil {
			log.Errorf("fail to delete content of directory '%s'", directory)
			return errors.Trace(err)
		}
	}

	// generate server files
	log.Debug("Calling Go Swagger code to generate API files")

	err = a.goSwaggerGenerate(path)
	if err != nil {
		log.Error("Fail to generate server files")
		return errors.Trace(err)
	}

	log.Debug("API files successfully generated using Go Swagger")

	return nil
}

// goSwaggerMixin will call go-swagger code to merge api yaml files.
func (a APIType) goSwaggerMixin(path string, primaryFile string, mixinFiles []string) ([]string, error) {
	log.Debugf("Starting %s  - %s", a.GetName(), helpers.GetCurrentFuncName())

	mixinSpec := commands.MixinSpec{
		Compact:         false,
		Output:          flags.Filename(filepath.Join(path, mergeYamlFileName)),
		KeepSpecOrder:   true,
		Format:          "yaml",
		IgnoreConflicts: false,
	}

	return mixinSpec.MixinFiles(primaryFile, mixinFiles, nil)
}

// goSwaggerGenerate will generate all app files as go-swagger binary will.
func (a APIType) goSwaggerGenerate(path string) error {
	log.Debugf("Starting %s  - %s", a.GetName(), helpers.GetCurrentFuncName())

	execDirectory, err := helpers.GetExecutableDirectory()
	if err != nil {
		log.Error("Fail to get Gouick directory")
		return errors.Trace(err)
	}

	generateCmd := commands.Generate{
		Server: &generate.Server{},
	}
	generateCmd.Server.Shared.Spec = flags.Filename(filepath.Join(path, mergeYamlFileName))
	generateCmd.Server.ServerPackage = "restapi"
	generateCmd.Server.Models.ModelPackage = "models"
	generateCmd.Server.Operations.APIPackage = "operations"
	generateCmd.Server.Shared.StrictResponders = false
	generateCmd.Server.Shared.TemplateDir = flags.Filename(filepath.Join(execDirectory, "templates/go-swagger/app/templates/"))
	generateCmd.Server.Shared.ConfigFile = flags.Filename(filepath.Join(execDirectory, "templates/go-swagger/app/config.yaml"))

	return generateCmd.Server.Execute(nil)
}
